import importlib
from cloud_storage_slim import CloudStorage


class GoogleCloudStorage(CloudStorage):
    def __init__(self) -> None:
        storage_module = importlib.import_module("google.cloud.storage")
        self.storage_client = storage_module.Client()

    def download(self, bucket_name, remote_blob_path, local_blob_path):
        bucket = self.storage_client.bucket(bucket_name)
        blob = bucket.blob(remote_blob_path)

        blob.download_to_filename(local_blob_path)

    def upload(self, bucket_name, local_blob_path, remote_blob_path):
        bucket = self.storage_client.bucket(bucket_name)
        blob = bucket.blob(remote_blob_path)

        blob.upload_from_filename(local_blob_path)

    def list_blobs(self, bucket_name, pattern):
        bucket = self.storage_client.get_bucket(bucket_name)
        blobs = bucket.list_blobs(prefix=pattern)
        blobs_list = list(blobs)
        blobs_list_names = [blob.name for blob in blobs_list]
        return blobs_list_names

    def get_first_blob(self, bucket_name, pattern):
        bucket = self.storage_client.get_bucket(bucket_name)
        blobs = bucket.list_blobs(prefix=pattern)
        blobs_list = list(blobs)
        if len(blobs_list) == 0:
            return None
        return blobs_list[0].name



import boto3
from cloud_storage_slim import CloudStorage


class AmazonS3Storage(CloudStorage):
    def __init__(self):
        self.s3_client = boto3.client("s3")

    def download(self, bucket_name, remote_blob_path, local_blob_path):
        self.s3_client.download_file(bucket_name, remote_blob_path, local_blob_path)

    def upload(self, bucket_name, local_blob_path, remote_blob_path):
        self.s3_client.upload_file(local_blob_path, bucket_name, remote_blob_path)

    def list_blobs(self, bucket_name, pattern):
        response = self.s3_client.list_objects_v2(Bucket=bucket_name, Prefix=pattern)
        return [item["Key"] for item in response.get("Contents", [])]

    def get_first_blob(self, bucket_name, pattern):
        blobs_list = self.list_blobs(bucket_name, pattern)
        return blobs_list[0] if blobs_list else None



import os
import unittest
from cloud_storage_slim import utils

class TestCloudStorageSlim(unittest.TestCase):

    def test_check_dest_local_file(self):
        # Test with a valid local file path
        valid_local_file_path = os.path.abspath(__file__)
        assert utils.check_dest_local_file(valid_local_file_path) == True

        # Test with a local file path that starts with "file://"
        file_uri_local_file_path = f"file:///{valid_local_file_path}"
        assert utils.check_dest_local_file(file_uri_local_file_path) == True

        # Test with a local file path that is actually a directory
        directory_local_file_path = os.path.dirname(valid_local_file_path)
        assert utils.check_dest_local_file(directory_local_file_path) == False

        # Test with a local file path that parent folder does not exist
        non_exist_parent_local_file_path = f"{directory_local_file_path}/non_exist_parent_folder/random-file.txt"
        assert utils.check_dest_local_file(non_exist_parent_local_file_path) == False

if __name__ == "__main__":
    unittest.main()



import importlib
import os
import logging
from cloud_storage_slim import CloudStorage


class AzureStorage(CloudStorage):
    def __init__(self) -> None:
        self.azure_storage_blob = importlib.import_module("azure.storage.blob")
        self.azure_identity = importlib.import_module("azure.identity")

        logging.basicConfig(level=logging.WARNING)
        azure_logger = logging.getLogger("azure")
        azure_logger.setLevel(logging.WARNING)

        az_account_name = os.environ.get("AZURE_STORAGE_ACCOUNT_NAME")
        az_account_key = os.environ.get("AZURE_STORAGE_ACCOUNT_KEY")
        connection_string = f"DefaultEndpointsProtocol=https;AccountName={az_account_name};AccountKey={az_account_key};EndpointSuffix=core.windows.net"
        self.blob_service_client = (
            self.azure_storage_blob.BlobServiceClient.from_connection_string(
                connection_string
            )
        )

    def download(self, container_name, remote_blob_path, local_blob_path):
        blob_client = self.blob_service_client.get_blob_client(
            container=container_name, blob=remote_blob_path
        )

        with open(local_blob_path, "wb") as download_file:
            download_file.write(blob_client.download_blob().readall())

    def upload(self, container_name, local_blob_path, remote_blob_path):
        blob_client = self.blob_service_client.get_blob_client(
            container=container_name, blob=remote_blob_path
        )

        with open(local_blob_path, "rb") as data:
            blob_client.upload_blob(data, overwrite=True)

    def list_blobs(self, container_name, pattern):
        container_client = self.blob_service_client.get_container_client(container_name)
        blobs_list = list(container_client.list_blobs(name_starts_with=pattern))
        blobs_list_names = [blob.name for blob in blobs_list]
        return blobs_list_names

    def get_first_blob(self, container_name, pattern):
        container_client = self.blob_service_client.get_container_client(container_name)
        blobs_list = list(container_client.list_blobs(name_starts_with=pattern))

        if len(blobs_list) == 0:
            return None

        return blobs_list[0].name



import os
import uuid
import shutil
import importlib.util
from cloud_storage_slim.utils import (
    parse_path_uri,
    check_scheme,
    check_source_local_file,
    check_dest_local_file,
)


class CloudStorage:
    def download(self, bucket_name, remote_blob_path, local_blob_path):
        raise NotImplementedError

    def upload(self, bucket_name, local_blob_path, remote_blob_path):
        raise NotImplementedError

    def list_blobs(self, bucket_name, pattern):
        raise NotImplementedError

    def get_first_blob(self, bucket_name, pattern):
        raise NotImplementedError


class CloudStorageSlim:
    def __init__(self) -> None:
        package_spec = importlib.util.find_spec("dotenv")
        if package_spec is not None:
            dotenv_module = importlib.util.module_from_spec(package_spec)
            package_spec.loader.exec_module(dotenv_module)
            dotenv_module.load_dotenv()

        self.gcs_client = None
        self.az_client = None
        self.oss_client = None
        self.s3_client = None

    def _setup_tmp_workspace(self):
        tmp_workspace_folder_path = os.path.join(
            os.path.expanduser("~"), ".cloud_storage_slim"
        )
        if not os.path.exists(tmp_workspace_folder_path):
            os.makedirs(tmp_workspace_folder_path)
        return tmp_workspace_folder_path

    def _teardown_tmp_workspace(self):
        tmp_workspace_folder_path = os.path.join(
            os.path.expanduser("~"), ".cloud_storage_slim"
        )
        if os.path.exists(tmp_workspace_folder_path):
            shutil.rmtree(tmp_workspace_folder_path)

    def _get_client(self, scheme) -> CloudStorage:
        if scheme == "gs" or scheme == "gcs":
            if self.gcs_client is None:
                from .google_cloud_storage import GoogleCloudStorage

                self.gcs_client = GoogleCloudStorage()
            return self.gcs_client
        elif scheme == "az":
            if self.az_client is None:
                from .azure_storage import AzureStorage

                self.az_client = AzureStorage()
            return self.az_client
        elif scheme == "oss":
            if self.oss_client is None:
                from .alibaba_cloud_oss import AlibabaCloudOSS

                self.oss_client = AlibabaCloudOSS()
            return self.oss_client
        elif scheme == "s3":
            if self.s3_client is None:
                from .amazon_s3 import AmazonS3Storage

                self.s3_client = AmazonS3Storage()
            return self.s3_client
        else:
            raise ValueError(f"Unknown scheme: {scheme}")

    def destroy(self):
        self._teardown_tmp_workspace()
        self.gcs_client = None
        self.az_client = None
        self.oss_client = None

    def _copy_local_to_remote(self, source_path, dest_path):
        local_blob_path = os.path.abspath(source_path)
        scheme, bucket_name, blob_path = parse_path_uri(dest_path)
        client = self._get_client(scheme)
        client.upload(bucket_name, local_blob_path, blob_path)

    def _copy_remote_to_local(self, source_path, dest_path):
        scheme, bucket_name, blob_path = parse_path_uri(source_path)
        client = self._get_client(scheme)
        client.download(bucket_name, blob_path, dest_path)

    def _copy_remote_to_remote(self, source_path, dest_path, filter_options=None):
        source_scheme, source_bucket_name, source_blob_path = parse_path_uri(
            source_path
        )
        source_client = self._get_client(source_scheme)

        dest_scheme, dest_bucket_name, dest_blob_path = parse_path_uri(dest_path)
        dest_client = self._get_client(dest_scheme)

        tmp_workspace_folder_path = self._setup_tmp_workspace()

        unique_local_filename = str(uuid.uuid4())
        local_blob_path = os.path.join(tmp_workspace_folder_path, unique_local_filename)

        try:
            if filter_options is not None:
                include = filter_options.get("include", None)
                if include is not None:
                    search_path = f"{source_blob_path}{include}"
                    source_blob_path = source_client.get_first_blob(
                        source_bucket_name, search_path
                    )
                    if source_blob_path is None:
                        raise ValueError(
                            f"Cannot find blob with prefix {search_path} from {source_scheme}://{source_bucket_name}"
                        )

            source_client.download(
                source_bucket_name, source_blob_path, local_blob_path
            )

            dest_client.upload(dest_bucket_name, local_blob_path, dest_blob_path)
        finally:
            if os.path.exists(local_blob_path):
                os.remove(local_blob_path)

    def copyto(self, source_path, dest_path, filter_options=None):
        """
        limitaions: only support single file copy right now
        """
        if check_source_local_file(source_path):
            # source is local file
            if check_dest_local_file(dest_path):
                # dest is local file
                # local to local
                shutil.copyfile(source_path, dest_path)
            else:
                # dest is remote file
                check_scheme(dest_path)
                # local to remote
                self._copy_local_to_remote(source_path, dest_path)
        else:
            # source is remote file
            check_scheme(source_path)
            if check_dest_local_file(dest_path):
                # dest is local file
                # remote to local
                self._copy_remote_to_local(source_path, dest_path)
            else:
                # dest is remote file
                check_scheme(dest_path)
                # remote to remote
                self._copy_remote_to_remote(source_path, dest_path, filter_options)

    def ls(self, remote_path, include=""):
        scheme, bucket_name, blob_path = parse_path_uri(remote_path)
        client = self._get_client(scheme)
        return client.list_blobs(bucket_name, f"{blob_path}{include}")



import os
import re
import logging
from urllib.parse import urlparse

logger = logging.getLogger(__name__)

class InvalidPathError(Exception):
    """Exception raised when the input path is invalid."""
    pass


def check_scheme(path_uri):
    path_scheme = urlparse(path_uri).scheme
    if path_scheme == "":
        raise ValueError(f"Invalid path uri: {path_uri}")
    if ["gs", "s3", "gcs", "az", "oss"].count(path_scheme) == 0:
        raise ValueError(f"Unsupported scheme: {path_scheme}")


def check_source_local_file(input_path):
    return (os.path.exists(input_path) and os.path.isfile(input_path)) or urlparse(
        input_path
    ).scheme == "file"


def check_dest_local_file(input_path):
    if input_path.startswith("file://"):
        input_path = urlparse(input_path).path

    input_path = os.path.abspath(os.path.expanduser(input_path))

    parent_dir, filename = os.path.split(input_path)

    invalid_chars_pattern = r'[<>:"|?*\x00]' if os.name == 'nt' else r'[\x00]'

    if re.search(invalid_chars_pattern, filename):
        logger.info(f"filename: [{filename}] contains invalid characters: {invalid_chars_pattern}")
        return False

    if os.path.isdir(input_path):
        logger.info(f"Path: [{input_path}] is a directory, not a file")
        return False

    if not os.path.exists(parent_dir):
        logger.info(f"The parent directory of Path: [{input_path}] does not exist")
        return False

    if not os.access(parent_dir, os.W_OK):
        logger.info(f"The parent directory: [{parent_dir}] is not writable")
        return False

    return True


def parse_path_uri(storage_bucket):
    parsed_url = urlparse(storage_bucket)
    scheme = parsed_url.scheme
    bucket_name = parsed_url.netloc
    blob_path = parsed_url.path.lstrip("/")
    return scheme, bucket_name, blob_path



import os
import random
import unittest
from dotenv import load_dotenv

load_dotenv()

from cloud_storage_slim import CloudStorageSlim


class TestCloudStorageSlim(unittest.TestCase):
    def get_gs_test_file_path_prefix(self, prefix=""):
        test_bucket_gs = os.environ.get("TEST_BUCKET_GS")
        return os.path.join(f"gs://{test_bucket_gs}", prefix)

    def get_s3_test_file_path_prefix(self, prefix=""):
        test_bucket_s3 = os.environ.get("TEST_BUCKET_S3")
        return os.path.join(f"s3://{test_bucket_s3}", prefix)

    def get_az_test_file_path_prefix(self, prefix=""):
        test_bucket_az = os.environ.get("TEST_BUCKET_AZ")
        return os.path.join(f"az://{test_bucket_az}", prefix)

    def get_oss_test_file_path_prefix(self, prefix=""):
        test_bucket_oss = os.environ.get("TEST_BUCKET_OSS")
        return os.path.join(f"oss://{test_bucket_oss}", prefix)

    def get_local_test_file_path_prefix(self, prefix=""):
        return os.path.join(self.test_working_dir, prefix)

    def get_random_remote_file_path(self):
        random_scheme = random.choice(["gs", "az", "oss"])
        return self.get_dest_test_file_path(random_scheme, "local", random_scheme)

    def create_local_file(
        self, local_file_path, file_content="this is cloud_storage_slim test file"
    ):
        with open(local_file_path, "w") as file:
            file.write(file_content)

    def create_remote_file(self, scheme, file_content=""):
        file_name = f"test_file_from_{scheme}.txt"
        file_content = f"this is cloud_storage_slim test file from {scheme}"
        if scheme == "gs":
            local_file_path = os.path.join(
                self.get_local_test_file_path_prefix(), file_name
            )
            self.create_local_file(local_file_path, file_content)
            remote_file_path = os.path.join(
                self.get_gs_test_file_path_prefix(), file_name
            )
            self.cloud_storage_slim.copyto(local_file_path, remote_file_path)
            return remote_file_path
        elif scheme == "az":
            local_file_path = os.path.join(
                self.get_local_test_file_path_prefix(), file_name
            )
            self.create_local_file(local_file_path, file_content)
            remote_file_path = os.path.join(
                self.get_az_test_file_path_prefix(), file_name
            )
            self.cloud_storage_slim.copyto(self.test_file_path, remote_file_path)
            return remote_file_path
        elif scheme == "oss":
            local_file_path = os.path.join(
                self.get_local_test_file_path_prefix(), file_name
            )
            self.create_local_file(local_file_path, file_content)
            remote_file_path = os.path.join(
                self.get_oss_test_file_path_prefix(), file_name
            )
            self.cloud_storage_slim.copyto(self.test_file_path, remote_file_path)
            return remote_file_path
        elif scheme == "s3":
            local_file_path = os.path.join(
                self.get_local_test_file_path_prefix(), file_name
            )
            self.create_local_file(local_file_path, file_content)
            remote_file_path = os.path.join(
                self.get_s3_test_file_path_prefix(), file_name
            )
            self.cloud_storage_slim.copyto(self.test_file_path, remote_file_path)
            return remote_file_path

    def get_dest_test_file_path(self, scheme, source, dest):
        path_prefix = ""
        if scheme == "gs":
            path_prefix = self.get_gs_test_file_path_prefix()
        elif scheme == "s3":
            path_prefix = self.get_s3_test_file_path_prefix()
        elif scheme == "az":
            path_prefix = self.get_az_test_file_path_prefix()
        elif scheme == "oss":
            path_prefix = self.get_oss_test_file_path_prefix()
        elif scheme == "file" or scheme == "local":
            path_prefix = self.get_local_test_file_path_prefix()

        return os.path.join(path_prefix, f"test_file_{source}_{dest}.txt")

    def setUp(self):
        self.cloud_storage_slim = CloudStorageSlim()
        self.test_working_dir = os.path.join(os.getcwd(), ".cache")
        if not os.path.exists(self.test_working_dir):
            os.mkdir(self.test_working_dir)

        self.test_file_path = "test_file.txt"
        self.create_local_file(self.test_file_path)

    def tearDown(self):
        self.cloud_storage_slim.destroy()
        if os.path.exists(self.test_file_path):
            os.remove(self.test_file_path)

    def test_copyto_local_to_local(self):
        source_path = self.test_file_path
        dest_path = "test_file_copy_from_local.txt"
        self.cloud_storage_slim.copyto(source_path, dest_path)
        self.assertTrue(os.path.exists(dest_path))
        self.assertEqual(os.path.getsize(source_path), os.path.getsize(dest_path))

    def test_copyto_local_to_remote(self):
        source_path = self.test_file_path
        dest_path_gs = self.get_dest_test_file_path("gs", "local", "gs")
        dest_path_s3 = self.get_dest_test_file_path("s3", "local", "s3")
        dest_path_az = self.get_dest_test_file_path("az", "local", "az")
        dest_path_oss = self.get_dest_test_file_path("oss", "local", "oss")
        # gs
        self.cloud_storage_slim.copyto(source_path, dest_path_gs)
        list_blobs = self.cloud_storage_slim.ls(dest_path_gs)
        self.assertEqual(len(list_blobs), 1)
        
        # s3
        self.cloud_storage_slim.copyto(source_path, dest_path_s3)
        list_blobs = self.cloud_storage_slim.ls(dest_path_s3)
        self.assertEqual(len(list_blobs), 1)

        # az
        self.cloud_storage_slim.copyto(source_path, dest_path_az)
        list_blobs = self.cloud_storage_slim.ls(dest_path_az)
        self.assertEqual(len(list_blobs), 1)

        # oss
        self.cloud_storage_slim.copyto(source_path, dest_path_oss)
        list_blobs = self.cloud_storage_slim.ls(dest_path_oss)
        self.assertEqual(len(list_blobs), 1)

    def test_copyto_remote_to_remote(self):
        source_path_gs = self.create_remote_file("gs")
        source_path_az = self.create_remote_file("az")
        source_path_oss = self.create_remote_file("oss")

        dest_path_gs_az = self.get_dest_test_file_path("az", "gs", "az")
        self.cloud_storage_slim.copyto(source_path_gs, dest_path_gs_az)
        list_blobs = self.cloud_storage_slim.ls(dest_path_gs_az)
        self.assertEqual(len(list_blobs), 1)

        dest_path_gs_s3 = self.get_dest_test_file_path("s3", "gs", "s3")
        self.cloud_storage_slim.copyto(source_path_gs, dest_path_gs_s3)
        list_blobs = self.cloud_storage_slim.ls(dest_path_gs_s3)
        self.assertEqual(len(list_blobs), 1)

        dest_path_gz_oss = self.get_dest_test_file_path("oss", "gs", "oss")
        self.cloud_storage_slim.copyto(source_path_gs, dest_path_gz_oss)
        list_blobs = self.cloud_storage_slim.ls(dest_path_gz_oss)
        self.assertEqual(len(list_blobs), 1)

        dest_path_az_gs = self.get_dest_test_file_path("gs", "az", "gs")
        self.cloud_storage_slim.copyto(source_path_az, dest_path_az_gs)
        list_blobs = self.cloud_storage_slim.ls(dest_path_az_gs)
        self.assertEqual(len(list_blobs), 1)

        dest_path_oss_gs = self.get_dest_test_file_path("gs", "oss", "gs")
        self.cloud_storage_slim.copyto(source_path_oss, dest_path_oss_gs)
        list_blobs = self.cloud_storage_slim.ls(dest_path_oss_gs)
        self.assertEqual(len(list_blobs), 1)

    def test_copyto_remote_to_local(self):
        source_path_gs = self.create_remote_file("gs")
        source_path_s3 = self.create_remote_file("s3")
        source_path_az = self.create_remote_file("az")
        source_path_oss = self.create_remote_file("oss")

        dest_path_gz_local = self.get_dest_test_file_path("local", "gs", "local")
        self.cloud_storage_slim.copyto(source_path_gs, dest_path_gz_local)
        self.assertTrue(os.path.exists(dest_path_gz_local))

        dest_path_s3_local = self.get_dest_test_file_path("local", "s3", "local")
        self.cloud_storage_slim.copyto(source_path_s3, dest_path_s3_local)
        self.assertTrue(os.path.exists(dest_path_s3_local))        

        dest_path_az_local = self.get_dest_test_file_path("local", "az", "local")
        self.cloud_storage_slim.copyto(source_path_az, dest_path_az_local)
        self.assertTrue(os.path.exists(dest_path_az_local))

        dest_path_oss_local = self.get_dest_test_file_path("local", "oss", "local")
        self.cloud_storage_slim.copyto(source_path_oss, dest_path_oss_local)
        self.assertTrue(os.path.exists(dest_path_oss_local))

    def test_copyto_invalid_source_path(self):
        source_path = os.path.join(
            self.get_local_test_file_path_prefix(), "invalid_file.txt"
        )
        dest_path = self.get_random_remote_file_path()
        with self.assertRaises(Exception):
            self.cloud_storage_slim.copyto(source_path, dest_path)

    def test_copyto_invalid_dest_path(self):
        source_path = self.test_file_path

        dest_path_gs = "gs://invalid_bucket/invalid_file.txt"
        with self.assertRaises(Exception):
            self.cloud_storage_slim.copyto(source_path, dest_path_gs)

        dest_path_az = "az://invalid_container/invalid_file.txt"
        with self.assertRaises(Exception):
            self.cloud_storage_slim.copyto(source_path, dest_path_az)

        dest_path_oss = "oss://invalid_bucket/invalid_file.txt"
        with self.assertRaises(Exception):
            self.cloud_storage_slim.copyto(source_path, dest_path_oss)


if __name__ == "__main__":
    unittest.main()



import os
import importlib
from cloud_storage_slim import CloudStorage


class AlibabaCloudOSS(CloudStorage):
    def __init__(self) -> None:
        self.oss2 = importlib.import_module("oss2")
        self.endpoint = os.environ.get("OSS_ENDPOINT")
        self.auth = self.oss2.ProviderAuth(self.oss2.credentials.EnvironmentVariableCredentialsProvider())

    def download(self, bucket_name, remote_blob_path, local_blob_path):
        bucket = self.oss2.Bucket(self.auth, self.endpoint, bucket_name)
        bucket.get_object_to_file(remote_blob_path, local_blob_path)

    def upload(self, bucket_name, local_blob_path, remote_blob_path):
        bucket = self.oss2.Bucket(self.auth, self.endpoint, bucket_name)
        bucket.put_object_from_file(remote_blob_path, local_blob_path)

    def list_blobs(self, bucket_name, pattern):
        res_blob_list = []
        bucket = self.oss2.Bucket(self.auth, self.endpoint, bucket_name)
        for obj in self.oss2.ObjectIterator(bucket, prefix=pattern):
            if obj.key.startswith(pattern):
                res_blob_list.append(obj.key)
        return res_blob_list

    def get_first_blob(self, bucket_name, pattern):
        bucket = self.oss2.Bucket(self.auth, self.endpoint, bucket_name)
        for obj in self.oss2.ObjectIterator(bucket, prefix=pattern):
            if obj.key.startswith(pattern):
                return obj.key
        return None



from setuptools import setup, find_packages

setup(
    name="cloud_storage_slim",
    version="0.1.5",
    author="Ying Cai",
    author_email="i@caiying.me",
    description="A lightweight cloud storage package for popular cloud storage services.",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url="https://github.com/yingca1/cloud-storage-slim",
    packages=find_packages(),
    install_requires=[],
)
